package riomhaire.lti.core.business.builder;

public class ActionBuilderFactory {

    private ActionBuilderBuilder launchContentBuilder = ()->new NopBuilder();
    private ActionBuilderBuilder processLTIMessageBuilder = ()->new NopBuilder();
    private ActionBuilderBuilder processOidcRequestBuilder= ()->new NopBuilder();
    private ActionBuilderBuilder selectDeepLinkContentBuilder = ()->new NopBuilder();

    public ActionBuilderFactory(ActionBuilderBuilder launchContentBuilder, ActionBuilderBuilder processLTIMessageBuilder, ActionBuilderBuilder processOidcRequestBuilder, ActionBuilderBuilder selectDeepLinkContentBuilder) {
        this.launchContentBuilder = launchContentBuilder;
        this.processLTIMessageBuilder = processLTIMessageBuilder;
        this.processOidcRequestBuilder = processOidcRequestBuilder;
        this.selectDeepLinkContentBuilder = selectDeepLinkContentBuilder;
    }
    public ActionBuilderFactory(ActionBuilderBuilder launchContentBuilder){
        this.launchContentBuilder = launchContentBuilder;
        this.processLTIMessageBuilder = ()->new ProcessLtiMessageBuilder();
        this.processOidcRequestBuilder = ()->new ProcessOidcRequestBuilder();
        this.selectDeepLinkContentBuilder = ()->new NopBuilder();

    }

    public ActionBuilder getBuilder(Builder name)    {
        return switch (name) {
            case LaunchContentBuilder : yield launchContentBuilder.build();
            case ProcessLTIMessageBuilder: yield processLTIMessageBuilder.build();
            case ProcessOidcRequestBuilder: yield processOidcRequestBuilder.build();
            case SelectDeepLinkContentBuilder: yield selectDeepLinkContentBuilder.build();
        } ;
    }
}
