package riomhaire.lti.core.infrastructure.facades.clientregistration;

import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.SneakyThrows;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.DigestUtils;
import riomhaire.lti.core.model.ClientConfiguration;
import riomhaire.lti.core.model.interfaces.ClientRegistrationResolver;

import java.util.Optional;

@Service
public class RedisBasedClientRegistrationResolver implements ClientRegistrationResolver {
    @Autowired
    private RedisTemplate<String, String> template;

    final ObjectMapper objectMapper = new ObjectMapper();

    @SneakyThrows
    @Override
    public Optional<ClientConfiguration> lookupClient(String issuer, String clientId) {
        var hash =  DigestUtils.md5DigestAsHex((issuer+clientId).getBytes());
        var key  = String.format("lti-client-registration-%s",hash);
        var blob = template.opsForValue().get(key);

        if( blob != null) {
          ClientConfiguration client = objectMapper.readValue(blob,ClientConfiguration.class);
          return Optional.of(client);
        }
        return Optional.empty();
    }
}
